@@ -105,7 +105,7 @@ bool admin_error(PgSocket *admin, const char *fmt, ...)
 
 	log_error("%s", str);
 	if (admin)
-		res = send_pooler_error(admin, true, str);
+		res = send_pooler_error(admin, true, false, str);
 	return res;
 }
 
@@ -234,8 +234,13 @@ static bool admin_set(PgSocket *admin, const char *key, const char *val)
 	if (admin->admin_user) {
 		ok = set_config_param(key, val);
 		if (ok) {
+			PktBuf *buf = pktbuf_dynamic(256);
+			if (strstr(key, "_tls_") != NULL) {
+				if (!sbuf_tls_setup())
+					pktbuf_write_Notice(buf, "TLS settings could not be applied, still using old configuration");
+			}
 			snprintf(tmp, sizeof(tmp), "SET %s=%s", key, val);
-			return admin_ready(admin, tmp);
+			return admin_flush(admin, buf, tmp);
 		} else {
 			return admin_error(admin, "SET failed");
 		}
@@ -492,9 +497,9 @@ static bool admin_show_databases(PgSocket *admin, const char *arg)
 		return true;
 	}
 
-	pktbuf_write_RowDescription(buf, "ssissiisiiii",
+	pktbuf_write_RowDescription(buf, "ssissiiisiiii",
 				    "name", "host", "port",
-				    "database", "force_user", "pool_size", "reserve_pool",
+				    "database", "force_user", "pool_size", "min_pool_size", "reserve_pool",
 				    "pool_mode", "max_connections", "current_connections", "paused", "disabled");
 	statlist_for_each(item, &database_list) {
 		db = container_of(item, PgDatabase, head);
@@ -504,11 +509,12 @@ static bool admin_show_databases(PgSocket *admin, const char *arg)
 		cv.value_p = &db->pool_mode;
 		if (db->pool_mode != POOL_INHERIT)
 			pool_mode_str = cf_get_lookup(&cv);
-		pktbuf_write_DataRow(buf, "ssissiisiiii",
+		pktbuf_write_DataRow(buf, "ssissiiisiiii",
 				     db->name, db->host, db->port,
 				     db->dbname, f_user,
-				     db->pool_size,
-				     db->res_pool_size,
+				     db->pool_size >= 0 ? db->pool_size : cf_default_pool_size,
+				     db->min_pool_size >= 0 ? db->min_pool_size : cf_min_pool_size,
+				     db->res_pool_size >= 0 ? db->res_pool_size : cf_res_pool_size,
 				     pool_mode_str,
 				     database_max_connections(db),
 				     db->connection_count,
@@ -810,9 +816,10 @@ static bool admin_show_pools(PgSocket *admin, const char *arg)
 		admin_error(admin, "no mem");
 		return true;
 	}
-	pktbuf_write_RowDescription(buf, "ssiiiiiiiiis",
+	pktbuf_write_RowDescription(buf, "ssiiiiiiiiiis",
 				    "database", "user",
 				    "cl_active", "cl_waiting",
+				    "cl_cancel_req",
 				    "sv_active", "sv_idle",
 				    "sv_used", "sv_tested",
 				    "sv_login", "maxwait",
@@ -822,10 +829,11 @@ static bool admin_show_pools(PgSocket *admin, const char *arg)
 		waiter = first_socket(&pool->waiting_client_list);
 		max_wait = (waiter && waiter->query_start) ? now - waiter->query_start : 0;
 		pool_mode = pool_pool_mode(pool);
-		pktbuf_write_DataRow(buf, "ssiiiiiiiiis",
+		pktbuf_write_DataRow(buf, "ssiiiiiiiiiis",
 				     pool->db->name, pool->user->name,
 				     statlist_count(&pool->active_client_list),
 				     statlist_count(&pool->waiting_client_list),
+				     statlist_count(&pool->cancel_req_list),
 				     statlist_count(&pool->active_server_list),
 				     statlist_count(&pool->idle_server_list),
 				     statlist_count(&pool->used_server_list),
@@ -971,6 +979,8 @@ static bool admin_cmd_reload(PgSocket *admin, const char *arg)
 
 	log_info("RELOAD command issued");
 	load_config();
+	if (!sbuf_tls_setup())
+		log_error("TLS configuration could not be reloaded, keeping old configuration");
 	return admin_ready(admin, "RELOAD");
 }
 
@@ -1028,7 +1038,7 @@ static bool admin_cmd_resume(PgSocket *admin, const char *arg)
 			return admin_error(admin, "no such database: %s", arg);
 		if (!db->db_paused)
 			return admin_error(admin, "database %s is not paused", arg);
-		db->db_paused = 0;
+		db->db_paused = false;
 	}
 	return admin_ready(admin, "RESUME");
 }
@@ -1051,7 +1061,7 @@ static bool admin_cmd_suspend(PgSocket *admin, const char *arg)
 
 	log_info("SUSPEND command issued");
 	cf_pause_mode = P_SUSPEND;
-	admin->wait_for_response = 1;
+	admin->wait_for_response = true;
 	suspend_pooler();
 
 	g_suspend_start = get_cached_time();
@@ -1071,7 +1081,7 @@ static bool admin_cmd_pause(PgSocket *admin, const char *arg)
 	if (!arg[0]) {
 		log_info("PAUSE command issued");
 		cf_pause_mode = P_PAUSE;
-		admin->wait_for_response = 1;
+		admin->wait_for_response = true;
 	} else {
 		PgDatabase *db;
 		log_info("PAUSE '%s' command issued", arg);
@@ -1080,9 +1090,9 @@ static bool admin_cmd_pause(PgSocket *admin, const char *arg)
 			return admin_error(admin, "no such database: %s", arg);
 		if (db == admin->pool->db)
 			return admin_error(admin, "cannot pause admin db: %s", arg);
-		db->db_paused = 1;
+		db->db_paused = true;
 		if (count_db_active(db) > 0)
-			admin->wait_for_response = 1;
+			admin->wait_for_response = true;
 		else
 			return admin_ready(admin, "PAUSE");
 	}
@@ -1140,7 +1150,7 @@ static bool admin_cmd_disable(PgSocket *admin, const char *arg)
 	if (db->admin)
 		return admin_error(admin, "cannot disable admin db: %s", arg);
 
-	db->db_disabled = 1;
+	db->db_disabled = true;
 	return admin_ready(admin, "DISABLE");
 }
 
@@ -1162,7 +1172,7 @@ static bool admin_cmd_enable(PgSocket *admin, const char *arg)
 	if (db->admin)
 		return admin_error(admin, "cannot disable admin db: %s", arg);
 
-	db->db_disabled = 0;
+	db->db_disabled = false;
 	return admin_ready(admin, "ENABLE");
 }
 
@@ -1189,7 +1199,7 @@ static bool admin_cmd_kill(PgSocket *admin, const char *arg)
 	if (db == admin->pool->db)
 		return admin_error(admin, "cannot kill admin db: %s", arg);
 
-	db->db_paused = 1;
+	db->db_paused = true;
 	statlist_for_each_safe(item, &pool_list, tmp) {
 		pool = container_of(item, PgPool, head);
 		if (pool->db == db)
@@ -1216,11 +1226,11 @@ static bool admin_cmd_wait_close(PgSocket *admin, const char *arg)
 
 		       pool = container_of(item, PgPool, head);
 		       db = pool->db;
-		       db->db_wait_close = 1;
+		       db->db_wait_close = true;
 		       active += count_db_active(db);
 	       }
 	       if (active > 0)
-		       admin->wait_for_response = 1;
+		       admin->wait_for_response = true;
 	       else
 		       return admin_ready(admin, "WAIT_CLOSE");
        } else {
@@ -1232,9 +1242,9 @@ static bool admin_cmd_wait_close(PgSocket *admin, const char *arg)
 		       return admin_error(admin, "no such database: %s", arg);
 	       if (db == admin->pool->db)
 		       return admin_error(admin, "cannot wait in admin db: %s", arg);
-	       db->db_wait_close = 1;
+	       db->db_wait_close = true;
 	       if (count_db_active(db) > 0)
-		       admin->wait_for_response = 1;
+		       admin->wait_for_response = true;
 	       else
 		       return admin_ready(admin, "WAIT_CLOSE");
        }
@@ -1483,8 +1493,8 @@ bool admin_handle_client(PgSocket *admin, PktHdr *pkt)
  */
 bool admin_pre_login(PgSocket *client, const char *username)
 {
-	client->admin_user = 0;
-	client->own_user = 0;
+	client->admin_user = false;
+	client->own_user = false;
 
 	/* tag same uid as special */
 	if (pga_is_unix(&client->remote_addr)) {
@@ -1497,8 +1507,8 @@ bool admin_pre_login(PgSocket *client, const char *username)
 			&& strcmp("pgbouncer", username) == 0)
 		{
 			client->login_user = admin_pool->db->forced_user;
-			client->own_user = 1;
-			client->admin_user = 1;
+			client->own_user = true;
+			client->admin_user = true;
 			slog_info(client, "pgbouncer access from unix socket");
 			return true;
 		}
@@ -1511,7 +1521,7 @@ bool admin_pre_login(PgSocket *client, const char *username)
 	if (cf_auth_type == AUTH_ANY) {
 		if (strlist_contains(cf_admin_users, username)) {
 			client->login_user = admin_pool->db->forced_user;
-			client->admin_user = 1;
+			client->admin_user = true;
 			return true;
 		} else if (strlist_contains(cf_stats_users, username)) {
 			client->login_user = admin_pool->db->forced_user;
@@ -1529,7 +1539,7 @@ bool admin_post_login(PgSocket *client)
 		return true;
 
 	if (client->admin_user || strlist_contains(cf_admin_users, username)) {
-		client->admin_user = 1;
+		client->admin_user = true;
 		return true;
 	} else if (strlist_contains(cf_stats_users, username)) {
 		return true;
@@ -1555,7 +1565,7 @@ void admin_setup(void)
 
 	db->port = cf_listen_port;
 	db->pool_size = 2;
-	db->admin = 1;
+	db->admin = true;
 	db->pool_mode = POOL_STMT;
 	if (!force_user(db, "pgbouncer", ""))
 		die("no mem on startup - cannot alloc pgbouncer user");
@@ -1592,7 +1602,7 @@ void admin_setup(void)
 		die("admin welcome failed");
 
 	pool->welcome_msg = msg;
-	pool->welcome_msg_ready = 1;
+	pool->welcome_msg_ready = true;
 
 	msg = pktbuf_dynamic(128);
 	if (!msg)
@@ -1645,7 +1655,7 @@ void admin_pause_done(void)
 		if (!res)
 			disconnect_client(admin, false, "dead admin");
 		else
-			admin->wait_for_response = 0;
+			admin->wait_for_response = false;
 	}
 
 	if (statlist_empty(&admin_pool->active_client_list)
@@ -1673,7 +1683,7 @@ void admin_wait_close_done(void)
 		if (!res)
 			disconnect_client(admin, false, "dead admin");
 		else
-			admin->wait_for_response = 0;
+			admin->wait_for_response = false;
 	}
 }
 
