@@ -9,7 +9,8 @@ export PGDATA=$PWD/pgdata
 export PGHOST=127.0.0.1
 export PGPORT=6667
 export EF_ALLOW_MALLOC_0=1
-export LANG=C
+export LC_ALL=C
+export POSIXLY_CORRECT=1
 
 BOUNCER_LOG=test.log
 BOUNCER_INI=test.ini
@@ -82,10 +83,14 @@ if ! $use_unix_sockets; then
 	echo 'admin_users = pgbouncer' >> test.ini
 fi
 
+MAX_PASSWORD=$(sed -n $SED_ERE_OP 's/#define MAX_PASSWORD[[:space:]]+([0-9]+)/\1/p' ../include/bouncer.h)
+long_password=$(printf '%*s' $(($MAX_PASSWORD - 1)) | tr ' ' 'a')
+
 # System configuration checks
 if ! grep -q "^\"${USER:=$(id -un)}\"" userlist.txt; then
 	cp userlist.txt userlist.txt.bak
 	echo "\"${USER}\" \"01234\"" >> userlist.txt
+	echo "\"longpass\" \"${long_password}\"" >> userlist.txt
 fi
 
 if test -n "$USE_SUDO"; then
@@ -124,7 +129,9 @@ rm -rf $PGDATA
 if [ ! -d $PGDATA ]; then
 	mkdir $PGDATA
 	initdb --nosync >> $PG_LOG 2>&1
-	sed $SED_ERE_OP -i "/unix_socket_director/s:.*(unix_socket_director.*=).*:\\1 '/tmp':" pgdata/postgresql.conf
+	if $use_unix_sockets; then
+		sed $SED_ERE_OP -i "/unix_socket_director/s:.*(unix_socket_director.*=).*:\\1 '/tmp':" pgdata/postgresql.conf
+	fi
 	cat >>pgdata/postgresql.conf <<-EOF
 	log_connections = on
 	EOF
@@ -168,6 +175,7 @@ psql -X -p $PG_PORT -d p0 -c "select * from pg_user" | grep pswcheck > /dev/null
 	psql -X -o /dev/null -p $PG_PORT -c "create user pswcheck with superuser createdb password 'pgbouncer-check';" p0 || exit 1
 	psql -X -o /dev/null -p $PG_PORT -c "create user someuser with password 'anypasswd';" p0 || exit 1
 	psql -X -o /dev/null -p $PG_PORT -c "create user maxedout;" p0 || exit 1
+	psql -X -o /dev/null -p $PG_PORT -c "create user longpass with password '$long_password';" p0 || exit 1
 	if $pg_supports_scram; then
 		psql -X -o /dev/null -p $PG_PORT -c "set password_encryption = 'md5'; create user muser1 password 'foo';" p0 || exit 1
 		psql -X -o /dev/null -p $PG_PORT -c "set password_encryption = 'md5'; create user muser2 password 'wrong';" p0 || exit 1
@@ -551,9 +559,67 @@ test_pool_size() {
 	test `docount p0` -eq 2 || return 1
 	test `docount p1` -eq 5 || return 1
 
+	# test reload (GH issue #248)
+	admin "set default_pool_size = 7"
+	test `docount p1` -eq 7 || return 1
+
 	return 0
 }
 
+test_min_pool_size() {
+	# make existing connections go away
+	psql -X -p $PG_PORT -d postgres -c "select pg_terminate_backend(pid) from pg_stat_activity where usename='bouncer'"
+	until test $(psql -X -p $PG_PORT -d postgres -tAq -c "select count(1) from pg_stat_activity where usename='bouncer'") -eq 0; do sleep 0.1; done
+
+	# default_pool_size=5
+	admin "set min_pool_size = 3"
+
+	cnt=`psql -X -p $PG_PORT -tAq -c "select count(1) from pg_stat_activity where usename='bouncer' and datname='p1'" postgres`
+	echo $cnt
+	test "$cnt" -eq 0 || return 1
+
+	# It's a bit tricky to get the timing of this test to work
+	# robustly: Full maintenance runs three times a second, so we
+	# need to wait at least 1/3 seconds for it to notice for sure
+	# that the pool is in use.  When it does, it will launch one
+	# connection per round, so we need to wait at least 3 * 1/3
+	# second before all the min pool connections are launched.
+	# Also, we need to keep the query running while this is
+	# happening so that the pool doesn't become momentarily
+	# unused.
+	psql -X -c "select pg_sleep(2)" p1 &
+	sleep 2
+
+	cnt=`psql -X -p $PG_PORT -tAq -c "select count(1) from pg_stat_activity where usename='bouncer' and datname='p1'" postgres`
+	echo $cnt
+	test "$cnt" -eq 3 || return 1
+}
+
+test_reserve_pool_size() {
+	# make existing connections go away
+	psql -X -p $PG_PORT -d postgres -c "select pg_terminate_backend(pid) from pg_stat_activity where usename='bouncer'"
+	until test $(psql -X -p $PG_PORT -d postgres -tAq -c "select count(1) from pg_stat_activity where usename='bouncer'") -eq 0; do sleep 0.1; done
+
+	# default_pool_size=5
+	admin "set reserve_pool_size = 3"
+
+	for i in {1..8}; do
+		psql -X -c "select pg_sleep(8)" p1 >/dev/null &
+	done
+	sleep 1
+	cnt=`psql -X -p $PG_PORT -tAq -c "select count(1) from pg_stat_activity where usename='bouncer' and datname='p1'" postgres`
+	echo $cnt
+	test "$cnt" -eq 5 || return 1
+
+	sleep 7  # reserve_pool_timeout + wiggle room
+
+	cnt=`psql -X -p $PG_PORT -tAq -c "select count(1) from pg_stat_activity where usename='bouncer' and datname='p1'" postgres`
+	echo $cnt
+	test "$cnt" -eq 8 || return 1
+
+	grep "taking connection from reserve_pool" $BOUNCER_LOG || return 1
+}
+
 test_max_db_connections() {
 	local users
 
@@ -829,6 +895,9 @@ test_password_server() {
 	# bad password from auth_file
 	psql -X -c "select 1" p4z && return 1
 
+	# long password from auth_file
+	psql -X -c "select 1" p4l || return 1
+
 	return 0
 }
 
@@ -844,6 +913,10 @@ test_password_client() {
 	PGPASSWORD=foo psql -X -U puser1 -c "select 1" p1 || return 1
 	# bad password
 	PGPASSWORD=wrong psql -X -U puser2 -c "select 2" p1 && return 1
+	# long password
+	PGPASSWORD=$long_password psql -X -U longpass -c "select 3" p1 || return 1
+	# too long password
+	PGPASSWORD=X$long_password psql -X -U longpass -c "select 4" p1 && return 1
 
 	# test with users that have an md5 password stored
 
@@ -1083,6 +1156,7 @@ test_no_user_md5_forced_user() {
 
 test_no_user_scram() {
 	$have_getpeereid || return 77
+	$pg_supports_scram || return 77
 
 	admin "set auth_type='scram-sha-256'"
 
@@ -1095,6 +1169,7 @@ test_no_user_scram() {
 
 test_no_user_scram_forced_user() {
 	$have_getpeereid || return 77
+	$pg_supports_scram || return 77
 
 	admin "set auth_type='scram-sha-256'"
 
@@ -1119,6 +1194,98 @@ test_no_user_auth_user() {
 	return 0
 }
 
+test_auto_database() {
+	psql -X -d p7 -c "select current_database()" || return 1
+	grep -F "registered new auto-database" $BOUNCER_LOG || return 1
+
+	return 0
+}
+
+test_cancel() {
+	case `uname` in MINGW*) return 77;; esac
+
+	psql -X -d p3 -c "select pg_sleep(20)" &
+	psql_pid=$!
+	sleep 1
+	kill -INT $psql_pid
+	wait $psql_pid
+	test $? -ne 0 || return 1
+	grep -F "canceling statement due to user request" $PG_LOG || return 1
+
+	return 0
+}
+
+# Test for waiting connections handling for cancel requests.
+#
+# The bug fixed by GH PR #542 was: When the connection pool is full,
+# cancel requests cannot get through (that is normal), but then when
+# unused connections close and pool slots are available, those are not
+# used for waiting cancel requests.
+test_cancel_wait() {
+	case `uname` in MINGW*) return 77;; esac
+
+	# default_pool_size=5
+	admin "set server_idle_timeout=2"
+
+	psql -X -d p3 -c "select pg_sleep(20)" &
+	psql_pid=$!
+	psql -X -d p3 -c "select pg_sleep(2)" &
+	psql -X -d p3 -c "select pg_sleep(2)" &
+	psql -X -d p3 -c "select pg_sleep(2)" &
+	psql -X -d p3 -c "select pg_sleep(2)" &
+	sleep 1
+
+	# This cancel must wait for a pool slot to become free.
+	kill -INT $psql_pid
+
+	wait $psql_pid
+
+	# Prior to the bug fix, the cancel would never get through and
+	# the first psql would simply run the full sleep and exit
+	# successfully.
+	test $? -ne 0 || return 1
+	grep -F "canceling statement due to user request" $PG_LOG || return 1
+
+	return 0
+}
+
+# Test that cancel requests can exceed the pool size
+#
+# Cancel request connections can use twice the pool size.  See also GH
+# PR #543.
+test_cancel_pool_size() {
+	case `uname` in MINGW*) return 77;; esac
+
+	# default_pool_size=5
+	admin "set server_idle_timeout=2"
+
+	psql -X -d p3 -c "select pg_sleep(20)" &
+	psql1_pid=$!
+	psql -X -d p3 -c "select pg_sleep(20)" &
+	psql2_pid=$!
+	psql -X -d p3 -c "select pg_sleep(20)" &
+	psql3_pid=$!
+	psql -X -d p3 -c "select pg_sleep(20)" &
+	psql4_pid=$!
+	psql -X -d p3 -c "select pg_sleep(20)" &
+	psql5_pid=$!
+	sleep 1
+
+	# These cancels requires more connections than the
+	# default_pool_size=5.
+	kill -INT $psql1_pid $psql2_pid $psql3_pid $psql4_pid $psql5_pid
+
+	wait $psql1_pid
+
+	# Prior to the change fix, the cancels would never get through
+	# and the psql processes would simply run the full sleep and
+	# exit successfully.
+	test $? -ne 0 || return 1
+	grep -F "canceling statement due to user request" $PG_LOG || return 1
+
+	return 0
+}
+
 testlist="
 test_show_version
 test_show
@@ -1135,6 +1302,8 @@ test_server_check_delay
 test_tcp_user_timeout
 test_max_client_conn
 test_pool_size
+test_min_pool_size
+test_reserve_pool_size
 test_max_db_connections
 test_max_user_connections
 test_online_restart
@@ -1163,6 +1332,10 @@ test_no_user_md5_forced_user
 test_no_user_scram
 test_no_user_scram_forced_user
 test_no_user_auth_user
+test_auto_database
+test_cancel
+test_cancel_wait
+test_cancel_pool_size
 "
 
 if [ $# -gt 0 ]; then
